severity = brutal
verbose = 1

exclude = Freenode

[-BuiltinFunctions::RequireBlockGrep]
[-BuiltinFunctions::RequireBlockMap]
[-BuiltinFunctions::ProhibitUniversalIsa]

[-CodeLayout::ProhibitParensWithBuiltins]

[-Community::LoopOnHash]
[-Community::Prototypes]

[-ControlStructures::ProhibitPostfixControls]
[-ControlStructures::ProhibitUnlessBlocks]

[-Documentation::PodSpelling]
[-Documentation::RequirePodAtEnd]
[-Documentation::RequirePodSections]

[-ErrorHandling::RequireCarping]

[-InputOutput::RequireBriefOpen]
[-InputOutput::RequireCheckedSyscalls]
[-InputOutput::RequireBracedFileHandleWithPrint]

[-Modules::ProhibitMultiplePackages]
[-NamingConventions::Capitalization]

[-References::ProhibitDoubleSigils]

[-RegularExpressions::RequireDotMatchAnything]
[-RegularExpressions::RequireExtendedFormatting]
[-RegularExpressions::RequireLineBoundaryMatching]

[-Subroutines::ProhibitExplicitReturnUndef]
[-Subroutines::ProhibitSubroutinePrototypes]
[-Subroutines::ProhibitUnusedPrivateSubroutines]
[-Subroutines::RequireArgUnpacking]
[-Subroutines::RequireFinalReturn]

[-TestingAndDebugging::RequireUseStrict]
[-TestingAndDebugging::RequireUseWarnings]

[-TryTiny::RequireUse]

[-ValuesAndExpressions::ProhibitAccessOfPrivateData]
[-ValuesAndExpressions::ProhibitConstantPragma]
[-ValuesAndExpressions::ProhibitFiletest_f]
[-ValuesAndExpressions::ProhibitMagicNumbers]
[-ValuesAndExpressions::ProhibitMixedBooleanOperators]
[-ValuesAndExpressions::ProhibitVersionStrings]
[-ValuesAndExpressions::RequireInterpolationOfMetachars]
[-ValuesAndExpressions::RequireNumericVersion]

[-Variables::ProhibitPunctuationVars]
[-Variables::ProhibitLoopOnHash]
[CodeLayout::RequireTidyCode]
perltidyrc = perltidy.rc

[Community::StrictWarnings]
extra_importers = Moo Moo::Role strictures Test2::V0
